/*
   SAA7110.c

   ZORAN ZR36120 module driver.
   Copyright (C) 1997  Vincent Labie (try ivsparis@pratique.fr with
				      for Vincent Labie in the suject)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include	"SAA7110.h"
#include	"ZoranI2C.h"

#define		SAA7110W	0x9c
#define		SAA7110R	0x9d

/**********************************************************************************************************************/
static const u8	TabSAA7110Init[0x3c]={	0x4c,0x3c,0x0d,0xef,0xbd,0xf0,0x00,0x00,	// from philips' pdf
					0xf8,0xf8,0x60,0x60,0x00,0x06,0x18,0x90,
					0x00,0x2c,0x40,0x46,0x42,0x1a,0xff,0xda,
					0xf0,0x8b,0x00,0x00,0x00,0x00,0x00,0x00,
/*static const u8	TabSAA7110Init[0x3c]={	0x50,0x30,0x00,0xe8,0xb6,0xf4,0x46,0x00,	// from meteor driver
					0x7f,0x7f,0x7f,0x7f,0x40,0x84,0x38,0x10,
					0x00,0x2c,0x40,0x40,0x34,0x0c,0xfb,0xd4,
					0xec,0x80,0x00,0x00,0x00,0x00,0x00,0x00,*/
					0xd9,0x17,0x40,0x41,0x80,0x41,0x80,0x4f,
					0xfe,0x01,0xcf,0x0f,0x03,0x01,0x81,0x03,
					0x44,0x75,0x01,0x8c,0x03	};

/**********************************************************************************************************************/
	BOOL	SAA7110Init(const ZORAN * pZoran)
{
	u8			Version;

	if (I2CRead(pZoran,SAA7110R,-1,&Version,1)<0)
		{ ERR(("Zoran%d: can't read SAA7110 version.\n",pZoran->IndexPCI));
		return(FALSE); }

	SAAWAR(("Zoran%d: init SAA7110 version %i ok.\n",pZoran->IndexPCI,Version));
	if (I2CWrite(pZoran,SAA7110W,0,TabSAA7110Init,sizeof(TabSAA7110Init))<0)
		{ ERR(("Zoran%d: can't init SAA7110.\n",pZoran->IndexPCI));
		return(FALSE); }

	return(TRUE);
}

/**********************************************************************************************************************/

static const u8	TabVideoInputCfg1Adr[]=	{0x06,0x20,0x21,0x22,0x2c,0x30,0x31,0x21};
static const u8	TabVideoInputCfg1[9][sizeof(TabVideoInputCfg1Adr)]={
					{0x00,0xd9,0x17,0x40,0x03,0x44,0x75,0x16},
					{0x00,0xd8,0x17,0x40,0x03,0x44,0x75,0x16},
					{0x00,0xba,0x07,0x91,0x03,0x60,0xb5,0x05},
					{0x00,0xb8,0x07,0x91,0x03,0x60,0xb5,0x05},
					{0x00,0x7c,0x07,0xd2,0x83,0x60,0xb5,0x03},
					{0x00,0x78,0x07,0xd2,0x83,0x60,0xb5,0x03},
					{0x80,0x59,0x17,0x42,0xa3,0x44,0x75,0x12},
					{0x80,0x9a,0x17,0xb1,0x13,0x60,0xb5,0x14},
					{0x80,0x3c,0x27,0xc1,0x23,0x44,0x75,0x21}
					};

static const u8	TabVideoInputCfg21[9]=	{0x16,0x16,0x05,0x05,0x03,0x03,0x12,0x14,0x21};

/**********************************************************************************************************************/
	int	SAA7110InputSetup(const ZORAN * pZoran,int Command,unsigned long arg)
{
	BOOL			Result		=0;
	u8			TabI2C[2];
	int			IndexCfg;
	int			bColorTrap	=FALSE;
	int			i;

	SAADBG(("SAA7110InputSetup Zoran%d: command=%i arg=0x%08lX.\n",pZoran->IndexPCI,Command,arg));
	switch(Command)
		{
		case ZORAN_CVBS1_NCT:
		case ZORAN_CVBS2_NCT:
		case ZORAN_CVBS3_NCT:
		case ZORAN_CVBS4_NCT:
		case ZORAN_CVBS5_NCT:
		case ZORAN_CVBS6_NCT:
			Command-=ZORAN_CVBS1_NCT-ZORAN_CVBS1;
			goto jump;
		case ZORAN_CVBS1:
		case ZORAN_CVBS2:
		case ZORAN_CVBS3:
		case ZORAN_CVBS4:
		case ZORAN_CVBS5:
		case ZORAN_CVBS6:
			bColorTrap=TRUE;
		case ZORAN_Y1C1:
		case ZORAN_Y2C2:
		case ZORAN_Y3C3:
jump:
			switch(arg)
				{
				case ZORAN_BW60HZ:
				case ZORAN_BW50HZ:
					bColorTrap=FALSE;
				case ZORAN_NTSC:
				case ZORAN_PAL:
				case ZORAN_SECAM:
					// reinit SAA to input
						// set input mode
					for(IndexCfg=0;IndexCfg<sizeof(TabVideoInputCfg1Adr);IndexCfg++)
						{
						ZRDBG(("Write SAA7110 adr 0x%02X=0x%02X\n",
							TabVideoInputCfg1Adr[IndexCfg],
							TabVideoInputCfg1[Command-ZORAN_CVBS1][IndexCfg]));
						if ((Result=I2CWrite(pZoran,SAA7110W,TabVideoInputCfg1Adr[IndexCfg],
							&TabVideoInputCfg1[Command-ZORAN_CVBS1][IndexCfg],1))<0) break;
						}
						// set input refs off clamp
					*TabI2C=TabVideoInputCfg21[Command-ZORAN_CVBS1];
					ZRDBG(("Write SAA7110 adr 0x%02X=0x%02X\n",0x21,*TabI2C));
					if ((Result=I2CWrite(pZoran,SAA7110W,0x21,TabI2C,1))<0) break;

					if (bColorTrap)		*TabI2C=0x00;	// color trap
					else			*TabI2C=0x80;
					ZRDBG(("Write SAA7110 adr 0x%02X=0x%02X\n",0x06,*TabI2C));
					if ((Result=I2CWrite(pZoran,SAA7110W,0x06,TabI2C,1))<0) break;

					if ((arg==ZORAN_NTSC)||(arg==ZORAN_BW60HZ))
									*TabI2C=0x40;
						if (arg==ZORAN_SECAM)	*TabI2C=0x30;
						else			*TabI2C=0x10;
					ZRDBG(("Write SAA7110 adr 0x%02X=0x%02X\n",0x0f,*TabI2C));
					if ((Result=I2CWrite(pZoran,SAA7110W,0x0f,TabI2C,1))<0) break;

					if ((arg==ZORAN_BW50HZ)||(arg==ZORAN_BW60HZ))
								*TabI2C=TabSAA7110Init[0x0c];
					else			*TabI2C=TabSAA7110Init[0x0c]|0x80;
					ZRDBG(("Write SAA7110 adr 0x%02X=0x%02X\n",0x0c,*TabI2C));
					if ((Result=I2CWrite(pZoran,SAA7110W,0x0c,TabI2C,1))<0) break;

					if (arg==ZORAN_SECAM)	*TabI2C=0x87;	// bit 1 => SECAM mode, 7 => VCR
					else			*TabI2C=0x86;	// doesn't warm when set...
					ZRDBG(("Write SAA7110 adr 0x%02X=0x%02X\n",0x0d,*TabI2C));
					if ((Result=I2CWrite(pZoran,SAA7110W,0x0d,TabI2C,1))<0) break;

					if (arg==ZORAN_NTSC)	*TabI2C=0x2c;	// chrominance gain ref val
					else			*TabI2C=0x59;
					ZRDBG(("Write SAA7110 adr 0x%02X=0x%02X\n",0x11,*TabI2C));
					if ((Result=I2CWrite(pZoran,SAA7110W,0x11,TabI2C,1))<0) break;

					if ((arg==ZORAN_NTSC)||(arg==ZORAN_BW60HZ))
								*TabI2C=0x81;
					else			*TabI2C=0x9a;	// blanking pulses
					ZRDBG(("Write SAA7110 adr 0x%02X=0x%02X\n",0x2e,*TabI2C));
					if ((Result=I2CWrite(pZoran,SAA7110W,0x2e,TabI2C,1))<0) break;
					Result=arg;
					break;
				case ZORAN_AUTO:
						// set input mode
					for(IndexCfg=0;IndexCfg<sizeof(TabVideoInputCfg1Adr);IndexCfg++)
						if ((Result=I2CWrite(pZoran,SAA7110W,TabVideoInputCfg1Adr[IndexCfg],
							&TabVideoInputCfg1[Command-ZORAN_CVBS1][IndexCfg],1))<0) break;

					*TabI2C=TabVideoInputCfg21[Command-ZORAN_CVBS1];
					if ((Result=I2CWrite(pZoran,SAA7110W,0x21,TabI2C,1))<0) break;

					for (i=0;i<15000;i++)
						schedule();

					Result=I2CRead(pZoran,SAA7110R,-1,TabI2C,1);
					if (Result<0) break;
					SAADBG(("Zoran%d: I2C READ 1 =%i\n",pZoran->IndexPCI,*TabI2C));

					if (((*TabI2C)&0x03)==0)	// no color
						if ((*TabI2C)&(0x20))	// 60Hz
							{ SAAWAR(("Zoran%d: Channel auto select found BW60HZ\n",
								pZoran->IndexPCI));
							Result=SAA7110InputSetup(pZoran,Command,ZORAN_BW60HZ); }
						else	{ SAAWAR(("Zoran%d: Channel auto select found BW50HZ\n",
								pZoran->IndexPCI));
							Result=SAA7110InputSetup(pZoran,Command,ZORAN_BW50HZ); }

					if ((*TabI2C)&0x20)		// 60Hz
						Result=SAA7110InputSetup(pZoran,Command,ZORAN_NTSC);
					else	if (((*TabI2C)&0x03)==0x01)	// PAL/SECAM
							{ SAAWAR(("Zoran%d: Channel auto select found SECAM\n",
								pZoran->IndexPCI));
							Result=SAA7110InputSetup(pZoran,Command,ZORAN_SECAM); }
						else	{ SAAWAR(("Zoran%d: Channel auto select found PAL\n",
								pZoran->IndexPCI));
							Result=SAA7110InputSetup(pZoran,Command,ZORAN_PAL); }
					Result&=0x0000ffff;
					Result|=((u32)(*TabI2C))<<16;
					break;
				default:
					Result=-EINVAL;
					break;
				}
			break;
		default:
			Result=-EINVAL;
			break;
		}
	if ((Result<0)&&(Result!=-EINVAL))
		ERR(("Zoran%d: cant setup SAA7110.\n",pZoran->IndexPCI));
	return(Result);
}

/**********************************************************************************************************************/
typedef	struct
	{
	int			I2CAdr;
	char *			szText;
	}		SAA7110SETUP;

static SAA7110SETUP	TabSAA7110Setup[ZORAN_BRIGHTNESS-ZORAN_HUE+1]=
		{ {0x07,"HUE gain"},{0x11,"Chrominance gain"},{0x12,"Chrominance saturation"},
		{0x13,"Luminance contrast"},{0x19,"Luminance brightness"} };

/**********************************************************************************************************************/
	BOOL	SAA7110Setup(const ZORAN * pZoran,int Command,unsigned long arg)
{
	u8			Byte		=arg;
	int			Result;

	if ((Command<ZORAN_HUE)||(Command>ZORAN_BRIGHTNESS)) return(-EINVAL);
	Result=I2CWrite(pZoran,SAA7110W,TabSAA7110Setup[Command-ZORAN_HUE].I2CAdr,&Byte,1);
	if (Result<0)	ERR(("Zoran%d: can't set SAA7110 %s to %i.\n",pZoran->IndexPCI,
				TabSAA7110Setup[Command-ZORAN_HUE].szText,(int)Byte));
	else		SAADBG(("Zoran%d: set SAA7110 %s to %i.\n",pZoran->IndexPCI,
				TabSAA7110Setup[Command-ZORAN_HUE].szText,(int)Byte));
	return(Result);
}

